/*
 * abMalloc.h - safe and clean, heap checking malloc functions
 * 		for the Active Badge poller software
 * 		December 1993
 */


#ifndef _ABMALLOC_H
#define _ABMALLOC_H


#include <malloc.h>
#include "abdiagnose.h"

#define MALLOC(STRUCTURE,TYPE,NUMBER,FUNCTION)                           				\
    if (!(STRUCTURE = (TYPE *) malloc(sizeof(TYPE) * NUMBER))) {    					\
    	ab_fatal(stdout, "%s: failed to malloc for %d * %s\n", FUNCTION, NUMBER, "TYPE");               	\
    }									 				\
    /** malloc completed **/
    

#define MALLOCD(STRUCTURE,TYPE,NUMBER,FUNCTION)                           				\
    ab_message(TRUE,"%s: malloc %d bytes (%d * %d) at line %d",FUNCTION,NUMBER * sizeof(TYPE),          \
               NUMBER,sizeof(TYPE),__LINE__);                                                           \
    MALLOC(STRUCTURE,TYPE,NUMBER,FUNCTION);                                                             \
    /** malloc completed **/
    
#define REALLOC(STRUCTURE,TYPE,NUMBER,FUNCTION)                           				\
    if (!(STRUCTURE = (TYPE *) realloc((void *) STRUCTURE, sizeof(TYPE) * NUMBER))) {  			\
    	ab_fatal(1, "%s: failed to realloc for %d * %s\n", FUNCTION, NUMBER, "TYPE");               	\
    }									 				\
    /** realloc completed **/    

#define CALLOC(STRUCTURE,TYPE,NUMBER,FUNCTION)                           				\
    if (!(STRUCTURE = (TYPE *) calloc(NUMBER,sizeof(TYPE)))) {    					\
    	ab_fatal(1, "%s: failed to calloc for %d * %s\n", FUNCTION, NUMBER, "TYPE");               	\
    }									 				\
    /** calloc completed **/
    
#define CALLOCD(STRUCTURE,TYPE,NUMBER,FUNCTION)                           				\
    ab_message(TRUE,"%s: calloc %d bytes (%d * %d) at line %d",FUNCTION,NUMBER * sizeof(TYPE),          \
               NUMBER,sizeof(TYPE),__LINE__);                                                           \
    CALLOC(STRUCTURE,TYPE,NUMBER,FUNCTION);                                                             \
    /** calloc completed **/
    
   
#define FREE(STRUCTURE) 	                                            				\
    free ((char *) STRUCTURE);                                           				

#define FREED(STRUCTURE,FUNCTION)	                                       				\
    ab_message(TRUE,"free at %X in %s, line %d",STRUCTURE,FUNCTION, __LINE__);				\
    FREE(STRUCTURE);                                           				


#define SAFEMEMCPY(TO_ARRAY,TO_INDEX,FROM_ARRAY,FROM_INDEX,LENGTH,TO_SIZE,FROM_SIZE,TO_STRING,FROM_STRING)	\
{														\
    fprintf(stderr,"memcopy from %s[%d] to %s[%d], %d bytes, %s is %d bytes, %s is %d bytes\n",			\
	    FROM_STRING,FROM_INDEX,										\
	    TO_STRING,TO_INDEX,											\
	    LENGTH,												\
	    FROM_STRING,FROM_SIZE,										\
	    TO_STRING,TO_SIZE);											\
    if ((TO_SIZE - TO_INDEX) < LENGTH) {									\
	fprintf(stderr,"buffer overwrite of %d bytes onto %s\n",						\
		LENGTH - (TO_SIZE - TO_INDEX),									\
		TO_STRING);											\
        exit(1);												\
    }														\
    if ((FROM_SIZE - FROM_INDEX) < LENGTH) {									\
        fprintf(stderr,"buffer overread of %d bytes from %s, index is %d, size is %d\n",		        \
		LENGTH - (FROM_SIZE - FROM_INDEX),								\
		FROM_STRING,											\
		FROM_INDEX,											\
		FROM_SIZE);											\
        exit(1);												\
    }														\
    memcpy( &(/**/TO_ARRAY/**/[/**/TO_INDEX/**/]), &(/**/FROM_ARRAY/**/[/**/FROM_INDEX/**/]),LENGTH);		\
}
	

#endif







