#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include "abpoller_i.hh"
#include <iostream.h>

void
AB_Poller_i::challenge(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station, const ABng_Poller::AB_Poller::Badge_Id& badge, 
		       const ABng_Poller::AB_Poller::Octets& random, ABng_Poller::AB_Poller::Seq_Number seq)
{
     unsigned int		i;
     unsigned int		temp;
     int		checksum;
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<challenge>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
	  cout << "Badge (length): " << badge.length() << endl;
	  cout << "Random (length): " << random.length() << endl;
	  cout << "Sequence number: " << (int)seq << endl;
     }

     msg.length(4+1+badge.length()+1+random.length()+1+1);
     msg[0] = 0xFF;	//preamble
     msg[1] = station; // SensorID
     msg[2] = 0xC0; //packet type
     msg[3] = 1+badge.length()+1+random.length()+1; //block length
     msg[4] = badge.length(); //badge address length
     for(i=5;i<5+badge.length();i++)
     {
	  msg[i] = badge[i-5]; //badge address
     }

     temp = 5+badge.length();
     msg[temp] = random.length(); //cycles & sound length
     for(i=temp+1;i<temp+1+random.length();i++)
     {
	  msg[i] = random[i-temp-1]; //sound values
     }

     temp += 1+random.length();
     checksum = msg[2]&0x1F;
     for(i=3;i<temp;i++)
     {
	  checksum += msg[i];
     }
     checksum &= 0xFF;
     msg[temp] = checksum; //checksum
     msg[temp+1] = network;
     issue_command(msg);
     return;     
}

void
AB_Poller_i::sound(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station, const ABng_Poller::AB_Poller::Badge_Id& badge, 
		       const ABng_Poller::AB_Poller::Tones& _sound, ABng_Poller::AB_Poller::Seq_Number seq)
{
     unsigned int	i;
     unsigned int	temp;
     unsigned char	checksum;
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<sound>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
	  cout << "Badge (length): " << badge.length() << endl;
	  cout << "Sound (length): " << _sound.length() << endl;
	  cout << "Sequence number: " << (int)seq << endl;
     }

     msg.length(4+1+badge.length()+1+_sound.length()+1+1);
     msg[0] = 0xFF;	//preamble
     msg[1] = station; // SensorID
     msg[2] = 0xc2; //packet type
     msg[3] = 1+badge.length()+1+_sound.length()+1; //block length
     msg[4] = badge.length(); //badge address length
     for(i=5;i<5+badge.length();i++)
     {
	  msg[i] = badge[i-5]; //badge address
     }

     temp = 5+badge.length();
     msg[temp] = _sound.length(); //cycles & sound length
     for(i=temp+1;i<temp+1+_sound.length();i++)
     {
	  msg[i] = _sound[i-temp-1]; //sound values
     }

     temp += 1+_sound.length();
     checksum = 0;
     checksum += msg[2]&0x1F;
     for(i=3;i<temp;i++)
     {
	  checksum += msg[i];
     }
     msg[temp] = checksum; //checksum
     msg[temp+1] = network;
     issue_command(msg);

    
     return;     
}

void
AB_Poller_i::set_state(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station, const ABng_Poller::AB_Poller::Badge_Id& badge, 
		       ABng_Poller::AB_Poller::Byte status, const ABng_Poller::AB_Poller::Tones& sound, ABng_Poller::AB_Poller::Seq_Number seq)
{
     unsigned int		i;
     unsigned int		temp;
     unsigned char	checksum;
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<set_state>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
	  cout << "Badge (length): " << badge.length() << endl;
	  cout << "Status: " << (int)status << endl;
	  cout << "Sound (length): " << sound.length() << endl;
	  cout << "Sequence number: " << (int)seq << endl;
     }

     msg.length(4+1+badge.length()+1+sound.length()+1+1);
     msg[0] = 0xFF;	//preamble
     msg[1] = station; // SensorID
     msg[2] = 0xC3; //packet type
     msg[3] = 1+badge.length()+1+sound.length()+1; //block length
     msg[4] = badge.length(); //badge address length
     for(i=5;i<5+badge.length();i++)
     {
	  msg[i] = badge[i-5]; //badge address
     }

     temp = 5+badge.length();
     msg[temp] = status;
     for(i=temp+1;i<temp+1+sound.length();i++)
     {
	  msg[i] = sound[i-temp-1]; //sound values
     }

     temp += 1+sound.length();
     checksum = 0;
     checksum += msg[2]&0x1F;
     for(i=3;i<temp;i++)
     {
	  checksum += msg[i];
     }
     msg[temp] = checksum; //checksum
     msg[temp+1] = network;
     issue_command(msg);
     return;     
}

void
AB_Poller_i::get_state(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station, const ABng_Poller::AB_Poller::Badge_Id& badge, 
		       ABng_Poller::AB_Poller::Seq_Number seq)
{
     unsigned int		i;
     unsigned int		temp;
     unsigned char	checksum;
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<get_state>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
	  cout << "Badge (length): " << badge.length() << endl;
	  cout << "Sequence number: " << (int)seq << endl;
     }

     msg.length(4+1+badge.length()+1+1);
     msg[0] = 0xFF;	//preamble
     msg[1] = station; // SensorID
     msg[2] = 0xC4; //packet type
     msg[3] = 1+badge.length()+1; //block length
     msg[4] = badge.length(); //badge address length
     for(i=5;i<5+badge.length();i++)
     {
	  msg[i] = badge[i-5]; //badge address
     }

     temp = 5+badge.length();
     checksum = msg[2]&0x1F;
     for(i=3;i<temp;i++)
     {
	  checksum += msg[i];
     }
     msg[temp] = checksum; //checksum
     msg[temp+1] = network;

     issue_command(msg);
     return;     
}

void
AB_Poller_i::set_home(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station, const ABng_Poller::AB_Poller::Badge_Id& badge, 
		      const ABng_Poller::AB_Poller::Address& address, ABng_Poller::AB_Poller::Seq_Number seq)
{
     unsigned int		i;
     unsigned int		temp;
     unsigned char	checksum;
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<set_home>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
	  cout << "Badge (length): " << badge.length() << endl;
	  cout << "Address (length): " << address.length() << endl;
	  cout << "Sequence number: " << (int)seq << endl;
     }
     msg.length(4+1+badge.length()+1+1+address.length()+1+1);
     msg[0] = 0xFF;	//preamble
     msg[1] = station; // SensorID
     msg[2] = 0xC1; //packet type
     msg[3] = 1+badge.length()+1+1+address.length()+1; //block length
     msg[4] = badge.length(); //badge address length
     for(i=5;i<5+badge.length();i++)
     {
	  msg[i] = badge[i-5]; //badge address
     }

     temp = 5+badge.length();
     msg[temp++] = 1+address.length();
     msg[temp] = 0x00;
     
     for(i=temp+1;i<temp+1+address.length();i++)
     {
	  msg[i] = address[i-temp-1]; //sound values
     }

     temp += 1+address.length();
     checksum = msg[2]&0x1F;
     for(i=3;i<temp;i++)
     {
	  checksum += msg[i];
     }
     msg[temp] = checksum; //checksum
     msg[temp+1] = network;

     issue_command(msg);
     return;     
}

void 
AB_Poller_i::get_home(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station, const ABng_Poller::AB_Poller::Badge_Id& badge, 
		       ABng_Poller::AB_Poller::Seq_Number seq)
{
     unsigned int		i;
     unsigned int		temp;
     unsigned char	checksum;
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<get_home>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
	  cout << "Badge (length): " << badge.length() << endl;
	  cout << "Sequence number: " << (int)seq << endl;
     }
     msg.length(4+1+badge.length()+1+1);
     msg[0] = 0xFF;	//preamble
     msg[1] = station; // SensorID
     msg[2] = 0xC7; //packet type
     msg[3] = 1+badge.length()+1; //block length
     msg[4] = badge.length(); //badge address length
     for(i=5;i<5+badge.length();i++)
     {
	  msg[i] = badge[i-5]; //badge address
     }

     temp = 5+badge.length();
     checksum = msg[2]&0x1F;
     for(i=3;i<temp;i++)
     {
	  checksum += msg[i];
     }
     msg[temp] = checksum; //checksum
     msg[temp+1] = network;

     issue_command(msg);
     return;     
}

void
AB_Poller_i::reset_buffer(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station)
{
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<reset_buffer>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
     }

     msg.length(6);
     msg[0] = 0xFF;
     msg[1] = (char)station;
     msg[2] = 0xE0;
     msg[3] = 0x01;
     msg[4] = 0x00;
     msg[5] = network;

     issue_command(msg);
     return;
}

void
AB_Poller_i::reset_fifos(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station)
{
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<reset_fifos>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
     }

     msg.length(6);
     msg[0] = 0xFF;
     msg[1] = (char)station;
     msg[2] = 0xE1;
     msg[3] = 0x01;
     msg[4] = 0x00;
     msg[5] = network;

     issue_command(msg);
     return;
}

void
AB_Poller_i::get_station_state(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station)
{
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<get_station_state>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
     }

     msg.length(6);
     msg[0] = 0xFF;
     msg[1] = (char)station;
     msg[2] = 0xE2;
     msg[3] = 0x01;
     msg[4] = 0x00;
     msg[5] = network;

     issue_command(msg);
     return;
}

void 
AB_Poller_i::set_output(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station,
			ABng_Poller::AB_Poller::Byte output_value)
{
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<set_output>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
	  cout << "Output value: " << (int)output_value << endl;
     }

     msg.length(6);
     msg[0] = 0xFF;
     msg[1] = (char)station;
     /*This is only for ORL sensors*/
     if (_networks[network] == ORL)
     {
	  if (output_value != 0) msg[2] = 0xE3;
	  else msg[2] = 0xE4;
     }
     else msg[2] = 0xE3;
     msg[3] = 0x01;
     if (_networks[network] == ORL)
	  msg[4] = 0x00;
     else msg[4] = output_value;
     msg[5] = network;

     issue_command(msg);
     return;
}

void
AB_Poller_i::set_persistence(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station,
			     CORBA::Boolean per_mode_value)
{
     ABng_Poller::AB_Poller::Message	msg;

     if (_debug)
     {
	  cout << "<<set_persistence>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
	  cout << "Persistence mode value: " << (int)per_mode_value << endl;
     }

     msg.length(6);
     msg[0] = 0xFF;
     msg[1] = (char)station;
     /*This is only for ORL sensors*/
     if (_networks[network] == ORL)
     {
	  if (per_mode_value != 0) msg[2] = 0xE5;
	  else msg[2] = 0xE6;
     }
     else msg[2] = 0xE5;
     msg[3] = 0x01;
     if (_networks[network] == ORL)
	  msg[4] = 0x00;
     else msg[4] = per_mode_value;
     msg[5] = network;

     issue_command(msg);
     return;
}

void
AB_Poller_i::broadcast(ABng_Poller::AB_Poller::Network_Id network, ABng_Poller::AB_Poller::Station_Id station,
		       const ABng_Poller::AB_Poller::Octets& data)
{
     ABng_Poller::AB_Poller::Message	msg;
     unsigned char	checksum;
     unsigned int		i;

     if (_debug)
     {
	  cout << "<<broadcast>> called" << endl;
	  cout << "Network: " << (int)network << endl;
	  cout << "Station: " << (int)station << endl;
	  cout << "Data (length): " << data.length() << endl;
     }

     msg.length(3+data.length()+1+1);
     msg[0] = 0xFF;	//preamble
     msg[1] = station; // SensorID
     msg[2] = 0x00; //packet type
     for(i=3;i<3+data.length();i++)
     {
	  msg[i] = data[i-3]; //badge address
     }

     checksum = msg[2]&0x1F;
     for(i=3;i<3+data.length();i++)
     {
	  checksum += msg[i];
     }
     msg[3+data.length()] = checksum; //checksum
     msg[3+data.length()+1] = network;
     issue_command(msg);
}
