#include <unistd.h>
#include <ctype.h>
#include <algo.h>

#include "polconfig.hh"

void CPollerConfig::Init()
{
  debug = false;

  char *name = new char[MAX_LINE];  
  *name = '\0';

  gethostname(name,MAX_LINE);

  CurLine = 0;

  // Set defaults

  GlobalParams["CORBAInterface"] = "True";
  GlobalParams["SocketInterface"] = "True";
  GlobalParams["PidFile"]        = "/var/run/abngpoller.pid";
  GlobalParams["NSIORFile"]      = "/etc/NS.ior";
  GlobalParams["Watchdog"]       = "False";
  crope hname = crope("@") + crope(name);
  GlobalParams["Name"]           = hname;
  GlobalParams["Logging"]        = "False";
  GlobalParams["MinStation"]        = "1";
  GlobalParams["MaxStation"]        = "254";

  GlobalParams["DebugAll"] = "False";
  GlobalParams["DebugStartup"] = "False";
  GlobalParams["DebugEfficiency"] = "False";
  GlobalParams["DebugSerialComm"] = "False";
  GlobalParams["DebugSensors"] = "False";
  GlobalParams["DebugCORBAInterface"] = "False";
  GlobalParams["DebugSocketInterface"] = "False";
 

  DefLineParams["Description"]   = "";
  DefLineParams["Device"]        = "/dev/ttyS0";
  DefLineParams["Port"]          = "1234";
  DefLineParams["Type"]          = "Olivetti";


  
  ConfigFile.open(ConfigFileName.c_str(), ios::in | ios::out);
  if(!ConfigFile)
    {
      // throw
      cerr << "Can't open configuration file" << endl;
      exit(0);
    }

  delete [] name;
  
}

CPollerConfig::~CPollerConfig()
{
  int n;
  map<const crope, crope, ltstr> *line;

  if(ConfigFile.is_open())
    ConfigFile.close();
   
}

void CPollerConfig::ErrorMessage(char *str)
{
  cerr << CONFIG_ERROR << CurLine << " : " << str << endl;
}

void GlobalParamsDump( pair<const crope, crope> entry)
{
  cout << entry.first <<  " = " <<  entry.second << endl;
}

void CPollerConfig::Dump()
{
  int n;

  cout << "Dumping configuration:" << endl;
  cout << "--- Global parameters ---" << endl;
  for_each(GlobalParams.begin(),GlobalParams.end(),
	  GlobalParamsDump);

  for(n=0 ; n < SerialLines.size() ; n++)
    {
      cout << "--- Line " << n+1 << " parameters ---" << endl;
      for_each(SerialLines[n].begin(),SerialLines[n].end(),
	     GlobalParamsDump);
    }
}


int CPollerConfig::Read()
{
  char line[MAX_LINE], *l;
  char *lside,*rside,*result;
  int n,len;
  int SectionOpen = false;
  int EquFound = false;
  //  map<const crope, crope, ltstr> *tmpline;
  crope first,second;
  int LinesCount = 0;
  map<const crope,crope, ltstr> *workmap;
  
  while( ConfigFile.getline(line,MAX_LINE) )
    {
      CurLine++;
      // remove comments
      len = strlen(line);
      for(n = 0 ; n < len ; n++)
	if( line[n] == '#' )
	  {
	    line[n] = '\0';
	    break;
	  }
      if ( (line[0] == '\n') || (line[0] == '\0')) 
	continue;

      // trimm leading spaces
      l = line;
      for(n = 0 ; line[n] && isblank((int) line[n]) ; n++)
	l++;
      
      if( strstr(l,LINE_OPEN))
	{
	  if( SectionOpen )
	    {
	      ErrorMessage("nested sections not supported");
	      exit(1);
	    }
	  SectionOpen = true;
	  workmap = new map<const crope,crope, ltstr>;
	  *workmap = DefLineParams;
	  //	  tmpline = new map<const crope, crope, ltstr>;
	  LinesCount++;
	}
      else if ( strstr(l,LINE_CLOSE))
	{
	  if( !SectionOpen)
	    {
	      ErrorMessage("open section before closing it");
	      exit(1);
	    }
	  SectionOpen = false;
	  map<const crope,crope, ltstr> tmap = *workmap;
	  SerialLines.push_back( tmap );
	  delete workmap;
	}
      else // Global or line entry
	{
	  
	  
	  // split line into lside and rside
	  
	  lside = l;
	  if((result = strstr(lside,"="))==NULL)
	    {
	      ErrorMessage(" '=' expected");
	      exit(1);
	    }
	  rside = result + 1;
	  *result = '\0';

	  // trimm leading and trailing white characters
	  for(n=strlen(lside)-1 ; n> 0 ; n--)
	    if( isblank(lside[n]) )  lside[n]='\0';

	  while( isblank( *rside) )
	    rside ++;

	  if( strlen(lside) == 0)
	    {
	      ErrorMessage(" parameter name missing");
	      exit(1);
	    }
	  if( strlen(rside) == 0)
	    {
	      ErrorMessage(" empty value");
	      exit(1);
	    }
	  
	  if(SectionOpen)
	    {
	      if(debug)
		cout << "line "<< LinesCount <<  
		  " parameter: " << lside << "=" << rside << endl;
	      // check wether it is a valid parameter
	      first  = crope(lside);
	      second = crope(rside);
	      (*workmap)[first] = second;
	    }
	  else
	    {
	      if(debug)
		cout << "Global parameter: " << 
		  lside << "=" << rside << endl;
	      
	      // check wether it is a valid parameter
	      GlobalParams[crope(lside)]=crope(rside);
	    }	    
	}

    }
  ConfigFile.close();
}
