#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <termios.h>
#include <unistd.h>
#ifdef _DOS
#include <sys\timeb.h>
#endif

#include "serial.h"
#include "poller.h"
#include "abmalloc.h"


#ifndef TRUE
#define TRUE            1
#endif

#ifndef FALSE
#define FALSE           0
#endif




/*
  ---- commands sent to the station by the poller
*/


#define NULL_COMMAND		(unsigned char) 0xe0
#define CLEAR_FIFO		(unsigned char) 0xe1
#define STN_STATUS		(unsigned char) 0xe2
#define FIELD_ON		(unsigned char) 0xe3
#define FIELD_OFF		(unsigned char) 0xe4
#define PERSISTENCE_ON		(unsigned char) 0xe5
#define PERSISTENCE_OFF		(unsigned char) 0xe6

#define POLL                    (unsigned char) 0xef

/*****************************************************************************/


#include <pthread.h>

pthread_mutex_t msgqueue_m;  //  addMessage & pollerRX critical section
extern pthread_mutex_t orl_serial_m;
extern int MIN_STATION, MAX_STATION;


int _polls = 0;
int _stations_total = 0;
pthread_mutex_t polls_m, stations_m;

typedef enum {
    RXOk,
    RXData,
    RXError,
    RXStationGone,
} ReceiveStatus;

static PollerMessage          	*messages = (PollerMessage *) NULL;
static PollerMessage          	*lastMessage = (PollerMessage *) NULL;
NetworkList		*networks = (NetworkList *) NULL;

static long			goodPacketsRXd;
static long			badPacketsRXd;
static long			totalPacketsTXd;
static long			startTime;
static long			packetTypes[NUMBER_OF_TYPES];

/*
  functions static to this sourcefile
*/


#ifdef _DOS
static void 		sendCommand(NetworkList *, unsigned char, unsigned char);
static int  		addMessage(NetworkList *, StationList *);
static int		addStation(NetworkList *, unsigned char);
static int		deleteStation(NetworkList *, unsigned char);
static ReceiveStatus	storeByteRX(NetworkList *, int);
static int		drainStation(unsigned char, NetworkList *, StationList *);
void		pollAnUnknownStationOlivetti(NetworkList *);
void		pollAnUnknownStationORL(NetworkList *);
#else 
static void 		sendCommand();
static int  		addMessage();
static int		addStation();
static int		deleteStation();
static ReceiveStatus	storeByteRX();
static int		drainStation();
void		pollAnUnknownStationOlivetti();
void		pollAnUnknownStationORL();
#endif

static unsigned char	pollerDiagnose;

extern int NextNumber;

/*****************************************************************************/


static void
sendCommand(network, station, command)
    NetworkList		*network;
    unsigned char	station;
    unsigned char	command;
{
    int		waste;

    if( network->protocol == PROTO_ORL)
      {
	if(command == POLL)
	  return;
	waste = serialTXRXByte(network->fd, (unsigned char) STUFF);
	waste = serialTXRXByte(network->fd, (unsigned char) station);
	waste = serialTXRXByte(network->fd, (unsigned char) command);
	waste = serialTXRXByte(network->fd, (unsigned char) 1);
	waste = serialTXRXByte(network->fd, (unsigned char) 0);
      }
    else if(network->protocol == PROTO_OLIVETTI)
      {
	unsigned char msg[5];
	
	msg[0] = (unsigned char) STUFF;
	msg[1] = (unsigned char) station;
	msg[2] = (unsigned char) command;
	msg[3] = (unsigned char) 1;
	msg[4] = (unsigned char) 0;

	serialTX(network->fd,msg,5);

      }
}

/*****************************************************************************/

/*
  ---- maintain a list of messages received from stations on a per-network
  ---- basis.
*/



static int
addMessage(network, station)
    NetworkList		*network;
    StationList		*station;
{
    PollerMessage		*message, *new;
    static int		diagnose = FALSE;
    


    if (diagnose) fprintf(stderr, "Add message for stn.net %d.%d\n",
			  (station->station & 0xff), (network->number & 0xff));

    if (network == (NetworkList *) NULL) {
	fprintf(stderr, "poller.addMessage called with null network\n");
	goto false;
    }

    if (station == (StationList *) NULL) {
	fprintf(stderr, "poller.addMessage called with null ntation\n");
	goto false;
    }

    message = &(station->message);

    MALLOC(new,PollerMessage,1,"poller.addMessage");

    if (message->count > 255) {
	fprintf(stderr, "** WARNING ** - addMessage called with message of length %d, from net.stn %d.%d\n",
		message->count, (unsigned int) (network->number & 0xff), (unsigned int) (station->station & 0xff));
	message->count = 255;		/* we truncate the message */
    }
    new->count = message->count + 4; /* Stn + Type + Length + Body + Network*/

    /** format: <stn> <type> <length> <body ...> <network> **/

    /*
     * we confirm the checksum before adding the packet
     */
    

    {
	unsigned char	chsum = 0;
	int		good = FALSE;
	int		i;

	if (station->length > 0) {
	    chsum = (unsigned char) (station->type & 0xff);
	    chsum += (station->length & 0xff);	

	    for (i = 0; i < station->length - 1; i++) {
		chsum = (unsigned char) ((chsum + message->bytes[i]) & 0xff);
	    }

	    if ((chsum & 0xff) == (message->bytes[i] & 0xff)) {
		good = TRUE;
	    }
	}
	
	if (good) {
	    goodPacketsRXd++;
	    network->goodPacketsRXd++;
	    station->goodPacketsRXd++;
	}
	else {
	    badPacketsRXd++;
	    network->badPacketsRXd++;
	    station->badPacketsRXd++;

	    FREE(new);

	    if (diagnose) fprintf (stderr, "Bad checksum!\n");
	    goto false;
	}
    }	

    new->bytes[0] = station->station;
    new->bytes[1] = station->type;
    new->bytes[2] = station->length;

    memcpy((char *) &(new->bytes[3]), (char *) message->bytes, message->count);

    new->bytes[message->count + 3] = (unsigned char) (network->number & 0xff);

    (network->packetTypes[station->type])++;

    packetTypes[station->type]++;

    new->next = (PollerMessage *) NULL;
    if (messages == (PollerMessage *) NULL) {
	messages = new;
	lastMessage = new;
    }
    else {
	lastMessage->next = new;
	lastMessage = new;
    }

    return TRUE;

 false:
    return FALSE;   
}



static int
addStation(network, station)
    NetworkList		*network;
    unsigned char	station;
{
    StationList		*ptr, *new;

    for (ptr = network->stations; ptr != (StationList *) NULL; ptr=ptr->next) {
	if (ptr->station == station) {
	    break;
	}
    }

    if (ptr != (StationList *) NULL) {
	fprintf(stderr, "%s:addStation %d : already exists on network \"%s\"\n", "poller", station, network->name);
	return FALSE;
    }

    MALLOC(new,StationList,1,"poller.addStation");

    fprintf(stderr, "addStation %d network %d\n", station, network->number  );
    fflush(stderr);
    SetStations(GetStations()+1);

    new->station = station;
    new->type = 0;
    new->length = 0;
    new->message.count = 0;
    new->RXing = FALSE;

    new->nextIsType = FALSE;
    new->nextIsLength = FALSE;
    new->lastWasStuff = FALSE;
    new->timeouts = 0;
    new->timeoutBlocks = 0;

    new->firstTXMessage = (PollerMessage *) NULL;
    new->lastTXMessage = (PollerMessage *) NULL;

    new->goodPacketsRXd = 0;
    new->badPacketsRXd = 0;
    new->totalPacketsTXd = 0;

    new->startTime = 0; /** statistics not implemented **/

    new->next = network->stations;
    network->stations = new;
    network->liveStations[(int) (station & 0xff)] = TRUE;

    return TRUE;
}

static int
deleteStation(network, station)
    NetworkList		*network;
    unsigned char	station;
{
    StationList	**list, *ptr;

    list = &(network->stations);
    ptr = (StationList *) NULL;

    while (*list) {
	if ((*list)->station == station) {
	    ptr = (*list);
	    (*list) = (*list)->next;
	    break;
	}
	else {
	    list = &((*list)->next);
	}
    }

    if (ptr == (StationList *) NULL) {
	return FALSE;
    }

    FREE(ptr);

    fprintf(stderr, "deleteStation %d network %d\n", station, network->number);
    fflush(stderr);
    SetStations(GetStations()-1);

    network->liveStations[(int) (station & 0xff)] = FALSE;
     return TRUE;
}


/**********************************************************************/

static ReceiveStatus
storeByteRX(network, byte)
    NetworkList		*network;
    int			byte;
{
    StationList		*stn;
    static int		diagnose = FALSE;

    if (network->lastStationAddressed == (StationList *) NULL) {
	if (diagnose) fprintf(stderr, "store %02x for unknown stn\n", byte);
	return (RXOk);
    }

    stn = network->lastStationAddressed;

    if (diagnose) fprintf(stderr,"store %02x for stn %d\n",byte, stn->station);

    if (byte == TIMEOUT) {

	stn->timeouts++;

	if (stn->timeouts > MAX_TIMEOUTS) {

	    stn->timeoutBlocks++;
	    

	    if (stn->timeoutBlocks == MAX_TIMEOUT_BLOCKS) {
	    
		/** if (diagnose) **/ fprintf(stderr,"poller.storeByteRX deleting station %d\n", stn->station, stn->timeoutBlocks);

		deleteStation(network, stn->station);

		network->lastStationAddressed = (StationList *) NULL;

	    }

	    return (RXStationGone);

	}
	return(RXOk);
    }
    
    stn->timeouts = 0;

    if (stn->RXing) {

      if ( (byte == STUFF) && (stn->lastWasStuff) ) { /* 2nd STUFF byte */
    	     	
	    stn->lastWasStuff = FALSE;
           
	    if ((unsigned char) (stn->message.count & 0xff) == stn->length) {

		(void) addMessage(network, stn);

		stn->RXing = FALSE;
		return(RXOk);
	    }	  	           
	    return(RXData);
	}
       	
      if (stn->lastWasStuff) { /* Last was stuff, but this isn't */
           
	    if (diagnose) fprintf(stderr, "Got stuff then %02x ?\n", (byte & 0xff));
               
	    stn->RXing = FALSE;  /* abandon this message */
	    return(RXError);
	}
    
	if (byte == STUFF) {
	    stn->lastWasStuff = TRUE;
	}
                    	
	if (stn->nextIsType) { /** received the type byte **/
    	
	    if (diagnose) fprintf(stderr,"Type byte %02x from %d\n", (byte & 0xff), stn->station);
     	
	    stn->type = (unsigned char) (byte & 0xff);

	    stn->nextIsType = FALSE;
	
	    stn->nextIsLength = TRUE;
        
	    return(RXData);        
	}

	if (stn->nextIsLength) { /** received the length byte **/
    	
	    stn->length = (unsigned char) (byte & 0xff);

	    stn->message.count = 0;

	    stn->nextIsLength = FALSE;
            
	    if (stn->length == 0) { /** can't have zero length message **/
		stn->RXing = FALSE;
		return(RXError);
	    }

	    return(RXData);
	}
	
	/** received a byte in the body of the message **/

	
	if (stn->message.count >= POLLER_MESSAGE_LENGTH) {
	    fprintf(stderr, "%s.%s.%d: Memory overwrite - count is %d\n",
		    "poller.c", "storeByteRX", __LINE__, stn->message.count);
	    exit(1);
	}
	stn->message.bytes[stn->message.count] = (char) (byte & 0xff);

	
	stn->message.count++;

	if (((unsigned char) (stn->message.count & 0xff) == stn->length) && !(stn->lastWasStuff)) {

	    (void) addMessage(network, stn);
	               
	    stn->RXing = FALSE;	
	    return(RXOk);
	}

	if (stn->message.count >= 0xff) {
	    fprintf(stderr, "** WARNING ** - full size packet received, but message not complete\n");

	    (void) addMessage(network, stn);
	               
	    stn->RXing = FALSE;	
	    return(RXOk);
	}

	return(RXData);	     
    }
    
    /** not receiving **/

    if (byte == STUFF) { /** new message coming in **/

	if (diagnose) fprintf(stderr, "message coming in from %d\n", stn->station);

	stn->RXing = TRUE;	 
	stn->nextIsType = TRUE;
	stn->nextIsLength = FALSE;
	stn->lastWasStuff = FALSE; 

	stn->message.count = 0;
	      
	return(RXData);
    }
     
    if (byte == PAD) {
	if (diagnose) fprintf(stderr, "pad while not receiving from %d\n", stn->station);
	return(RXOk);
    }
    
    return(RXError);	
}


/*
  drain a station's FIFO. Finish with a STUFF transmission
*/

static int
drainStation(nextTX, network, station)
    unsigned char	nextTX;
    NetworkList		*network;
    StationList		*station;
{
    int			nextRX, junk;
    ReceiveStatus	response;

    do {
	nextRX = serialTXRXByte(network->fd, nextTX);
	if ((response = storeByteRX(network, nextRX)) == RXStationGone) {
	    if (nextTX == STUFF) {
		junk = serialTXRXByte(network->fd, NULL_STATION);
	    }
	    return FALSE;
	}

	nextTX = (nextTX == STUFF ? station->station : STUFF );
	/** maybe the station has gone away here **/
    } while ((response == RXData) || (nextRX == STUFF));

    if (nextTX != STUFF) { /* we can stop here */
	return TRUE;
    }

    nextRX = serialTXRXByte(network->fd, STUFF);

    if ((response = storeByteRX(network, nextRX)) == RXStationGone) {
	junk = serialTXRXByte(network->fd, NULL_STATION);
	return FALSE;
    }
    return TRUE;
}

/*
 * poll one unkown station on the given network to see if it's alive
 */

void
pollAnUnknownStationOlivetti(network)
    NetworkList		*network;
{
}


int
AddressUnknownStation(network)
    NetworkList		*network;
{

  /*  fprintf(stderr,"Addressing station %d/%d\n",(unsigned int)network->lastUnknownStation, (unsigned int) network->number); */
  if( network->protocol == PROTO_ORL)
    {
      int nextRX;
      ReceiveStatus	response;

      nextRX = serialTXRXByte(network->fd, (unsigned char) network->lastUnknownStation);
      response = storeByteRX(network, nextRX);

      network->lastStationAddressed = (StationList *) NULL;
      nextRX = serialTXRXByte(network->fd, STUFF);
      if(nextRX != TIMEOUT)
	return TRUE;
      else
	return FALSE;
    }
 else if(network->protocol == PROTO_OLIVETTI)
      {
	unsigned char msg[5];
	unsigned char retbuf[30];
	int result;

		
	msg[0] = (unsigned char) STUFF;
	msg[1] = (unsigned char) network->lastUnknownStation;
	msg[2] = (unsigned char) 0xE2;
	msg[3] = (unsigned char) 0x01;
	msg[4] = (unsigned char) 0x00;

	serialTX(network->fd,msg,5);
	result = serialRX(network->fd,retbuf,sizeof(retbuf));
	if( retbuf[1] == 0x62 )
	  return TRUE;
	else
	  return FALSE;
	    
      }
  return FALSE;
}


void
pollAnUnknownStationORL(network)
    NetworkList		*network;
{
    int			i;
    int			junk;


    if ((network->lastUnknownStation > MAX_STATION) || (network->lastUnknownStation == 0) || (network->liveStations[network->lastUnknownStation] == TRUE)) {

	if ((network->lastUnknownStation > MAX_STATION) || (network->lastUnknownStation == 0)) {
	    i = MIN_STATION;
	}
	else {
	    i = network->lastUnknownStation+1;
	}

	for (; i < (MAX_STATION + 1); i++) {
	    if (network->liveStations[i] == FALSE) {
		break;
	    }
	}
	if (i == (MAX_STATION + 1)) {
	    /** no unknown stations **/
	    return;
	}

	network->lastUnknownStation = i;
    }
 

    if (AddressUnknownStation(network)) { /* station exists   */  
	if (!(addStation(network, (unsigned char) network->lastUnknownStation))) {
	    fprintf(stderr, "poller: failed to add station %d on network %d (%s)\n", i, network->number, network->name);
	    exit(1); /* of course, maybe I won't exit here */
	}
		
	sendCommand(network, (unsigned char) (network->lastUnknownStation & 0xff), CLEAR_FIFO);
	sendCommand(network, (unsigned char) (network->lastUnknownStation & 0xff), PERSISTENCE_OFF);
	sendCommand(network, (unsigned char) (network->lastUnknownStation & 0xff), NULL_COMMAND);
	sendCommand(network, (unsigned char) (network->lastUnknownStation & 0xff), FIELD_OFF);
	sendCommand(network, (unsigned char) (network->lastUnknownStation & 0xff), CLEAR_FIFO);

	junk = serialTXRXByte(network->fd, STUFF);
    }

    junk = serialTXRXByte(network->fd, NULL_STATION);
    network->lastUnknownStation++;
    network->lastStationAddressed = (StationList *) NULL;
}

/**********************************************************************/

/*
 * ---- poller <-> serialDevice interface
 */


int
pollerAddSerialDevice(char *name,char *descr, int protocol)

{
    NetworkList	*ptr, *new;
    static int	first = TRUE;
    int		serialOpenResult;
    int		i;

    if ((name == (char *) 0) || (strlen(name) == 0)) {
      return -1;	/* silly name for a network */
    }

    if (first) {

	first = FALSE;
	goodPacketsRXd = 0;
	badPacketsRXd = 0;
	totalPacketsTXd = 0;


        startTime = 0;
	
	for (i = 0; i < NUMBER_OF_TYPES; i++) {
	  packetTypes[i] = 0; /* tally of the no. of packets of each type */
	}
    }

    for (ptr = networks; ptr != (NetworkList *) NULL; ptr = ptr->next) {
	if (strcmp(ptr->name, name) == 0) {
	    break;
	}
    }
    if (ptr != (NetworkList *) NULL) {
      printf("Network already initialised\n");
      return -1; /* already have a network of this name */
    }

    MALLOC(new,NetworkList,1,"poller.addSerialDevice");

    strcpy(new->name, name);
    strcpy(new->description,descr);
    new->protocol = protocol;
    new->stations = (StationList *) NULL;
    new->lastStationAddressed = (StationList *) NULL;

    memset((char *) new->liveStations, (char) 0, sizeof(new->liveStations));
    new->lastUnknownStation = 0;

    if ((serialOpenResult = serialOpen(name)) == -1) {

    	FREE(new);
    	
	return -1;
    }
    new->fd = (unsigned char) (serialOpenResult & 0xff) ;
    
    new->goodPacketsRXd = 0;
    new->badPacketsRXd = 0;
    new->totalPacketsTXd = 0;
	
    for (i = 0; i < NUMBER_OF_TYPES; i++) {
      new->packetTypes[i] = 0; /* tally of the no. of packets of each type */
    }

    new->next = networks;
    new->number = NextNumber;
    networks = new;

    return new->fd;
}



int
pollerDeleteSerialDevice(name)
    char	*name;
{
    NetworkList	*ptr, **list;

    fprintf(stderr, "pollerDeleteSerialDevice called \"%s\"\n", name);

    ptr = (NetworkList *) NULL;
    list = &networks;

    while (*list) {
	if (strcmp((*list)->name, name) == 0) {
	    ptr = (*list);
	    *list = (*list)->next;
	    break;
	}
	else {
	    list = &((*list)->next);
	}
    }

    if (ptr == (NetworkList *) NULL) {
	return FALSE;
    }

    /** must delete all stations on this network **/

    /** not done yet! **/

    exit(0);

    return FALSE;
}



int 
pollerPollORL(NetworkList *nPtr)
{
  StationList		*sPtr;
  int			nextRX, junk;
  ReceiveStatus	response;

 nextRX = serialTXRXByte(nPtr->fd, STUFF);
  if ((response = storeByteRX(nPtr, nextRX)) == RXStationGone) {
    junk = serialTXRXByte(nPtr->fd, NULL_STATION);
    goto false;
  }

  for (sPtr = nPtr->stations; sPtr; sPtr = sPtr->next) {
	  
    SetPolls(GetPolls()+1);

    nextRX = serialTXRXByte(nPtr->fd, sPtr->station);
    if ((response = storeByteRX(nPtr, nextRX)) == RXStationGone) {
      goto false;
    }
    nPtr->lastStationAddressed = sPtr;

    nextRX = serialTXRXByte(nPtr->fd, STUFF);
    if ((response = storeByteRX(nPtr, nextRX)) == RXStationGone) {
      junk = serialTXRXByte(nPtr->fd, NULL_STATION);
      goto false;
    }
    else if (response == RXData) {
      if (!(drainStation(sPtr->station, nPtr, sPtr))) {
	goto false;
      }
    }
  }
 false:
  return FALSE;
}

int 
pollerPollOlivetti(NetworkList *nPtr)
{
  StationList		*sPtr,*tmp;
  int result,j;
  unsigned char buf[255];
  
  for (sPtr = nPtr->stations; sPtr; sPtr = sPtr->next) 
    {
  
      nPtr->lastStationAddressed = sPtr;
      SetPolls(GetPolls()+1);
      sendCommand(nPtr,sPtr->station,POLL);
      usleep(30000);
      result = serialRX(nPtr->fd,buf,255);
      if(result > 0)
	{
	  printf("Received %d bytes from %d/%d: ",result,(int)sPtr->station,nPtr->number);
	  for(j=0 ; j < result ; j++)
	    {
	      storeByteRX(nPtr,buf[j]);
	      printf("%X ", 0xFF & buf[j]);
	    }
	    printf("\n");
	    sPtr->timeouts = 0;
	}
      if(result == 0)
	{
	  sPtr->timeouts++;
	  
	  if( sPtr->timeouts > MAX_TIMEOUTS*2)
	    {
	      sPtr->timeoutBlocks++;
	     
	      if( sPtr->timeoutBlocks == MAX_TIMEOUT_BLOCKS)
		{
		  //tmp = sPtr;
		  
		  //sPtr = sPtr->next;
		  //deleteStation(nPtr,tmp->station);
		  //break;
		}
	    }
	    
	  /*	  fprintf(stderr, "Station %d/%d inactive\n",(int)sPtr->station,nPtr->number);*/
	}
      if((result <=3) && (result >0) )
	{
	  fprintf(stderr, "Short message from %d/%d inactive\n",(int)sPtr->station,nPtr->number);
	}
      
    }

}


int
pollerPollNetworks(NetworkList *nPtr)
{
 

  static int l=POLL_UNKNOWN_INTERVAL;


  if(l-- <= 0)
    l=POLL_UNKNOWN_INTERVAL;

  if(l == 0)
    {
      //      if( nPtr->protocol == PROTO_ORL)
      pollAnUnknownStationORL(nPtr);	
	//else
	//if(nPtr->protocol == PROTO_OLIVETTI)
	//pollAnUnknownStationOlivetti(nPtr);
    }

  /* poll every station on this network */
  if( nPtr->protocol == PROTO_ORL)
    return pollerPollORL(nPtr);
  else
    if(nPtr->protocol == PROTO_OLIVETTI)
      return pollerPollOlivetti(nPtr);
  else
    {
      fprintf(stderr,"Unknown protocol: %d (pollerPollNetworks)\n",nPtr->protocol);
      return 0;
    }


  
  return TRUE;

}



PollerMessage *
pollerRX()
{
    PollerMessage		*mPtr;
    /* return a message if one has been received */


    if (messages == (PollerMessage *) NULL) {
      return (PollerMessage *) NULL;
    }
    else {
	mPtr = messages;
	messages = messages->next;
	return mPtr;
    }
}

int
pollerTX(network_number, station, count, message)
    unsigned char	network_number;
    unsigned char	station;
    int			count;
    unsigned char	*message;
{
    NetworkList		*nPtr;
    StationList		*sPtr;
    int			i;
    int			nextRX, junk;
    ReceiveStatus	response;

    for (nPtr = networks; nPtr != (NetworkList *) NULL; nPtr = nPtr->next) {
	if (network_number == nPtr->number) {
	    break;
	}
    }
    if (nPtr == (NetworkList *) NULL) {
	return FALSE;
    }
    
    for (sPtr = nPtr->stations; sPtr != (StationList *) NULL; sPtr=sPtr->next){
	if (sPtr->station == station) {
	    break;
	}
    }
    if (sPtr == (StationList *) NULL) {
	return FALSE;
    }

    nextRX = serialTXRXByte(nPtr->fd, STUFF);	  /* address the station ... */
    if ((response = storeByteRX(nPtr, nextRX)) == RXStationGone) {
      junk = serialTXRXByte(nPtr->fd, NULL_STATION); /* clean up */
	return FALSE;
    }

    nextRX = serialTXRXByte(nPtr->fd, sPtr->station);
    if ((response = storeByteRX(nPtr, nextRX)) == RXStationGone) {
	return FALSE;
    }
    nPtr->lastStationAddressed = sPtr;

    for (i = 0; i < count; i++) { /* transmit each byte in the message ... */
    	nextRX = serialTXRXByte(nPtr->fd, message[i]);
	if ((response = storeByteRX(nPtr, nextRX)) == RXStationGone) {
	  junk = serialTXRXByte(nPtr->fd, STUFF); /* clean up */
	    junk = serialTXRXByte(nPtr->fd, NULL_STATION);
	    nPtr->lastStationAddressed = (StationList *) NULL;
	    return FALSE;
	}
	if (message[i] == STUFF) { /** double-up STUFF bytes on the wire **/
	    nextRX = serialTXRXByte(nPtr->fd, message[i]);
	    if ((response = storeByteRX(nPtr, nextRX)) == RXStationGone) {
	      junk = serialTXRXByte(nPtr->fd, STUFF); /* clean up */
		junk = serialTXRXByte(nPtr->fd, NULL_STATION);
		nPtr->lastStationAddressed = (StationList *) NULL;
		return FALSE;
	    }
	}
	printf("%d ", response);
    }
    printf("\n");
	
/* after TXing, maybe need to complete partially RX'd message */

    if (response == RXData) {
        if (!(drainStation(STUFF, nPtr, sPtr))) {
	    return FALSE;
	}
    }
    return TRUE;
}


int
pollerInitialiseNetwork(name)
    char		*name;
{
    NetworkList		*nPtr;
    StationList *sPtr;


    if ((name == (char *) 0) || (strlen(name) == 0)) {
return FALSE;	/* silly name for a network */
    }

    for (nPtr = networks; nPtr != (NetworkList *) NULL; nPtr = nPtr->next) {
	if (strcmp(nPtr->name, name) == 0) {
	    break;
	}
    }
    if (nPtr == (NetworkList *) NULL) {
return FALSE; /* no network of this name*/
    }

    if( nPtr->protocol == PROTO_ORL )
      {
	return pollerInitialiseORL(nPtr);
      }
    else if ( nPtr->protocol == PROTO_OLIVETTI )
      {
	return pollerInitialiseOlivetti(nPtr);
      }
    else
      {
	// Unknown protocol
	fprintf(stderr,"Unknown protocol code %d\n",nPtr->protocol);
	return FALSE;
      }

/* initialise every station found */

    for (sPtr = nPtr->stations; sPtr; sPtr=sPtr->next) {
	sendCommand(nPtr, sPtr->station, CLEAR_FIFO);
	sendCommand(nPtr, sPtr->station, PERSISTENCE_OFF);
	sendCommand(nPtr, sPtr->station, NULL_COMMAND);
	sendCommand(nPtr, sPtr->station, FIELD_OFF);
	sendCommand(nPtr, sPtr->station, CLEAR_FIFO);
    }

}


int
pollerInitialiseORL(NetworkList *nPtr)
{
  //    StationList		*sPtr;
    int			i;
    int			nextRX, junk;

    /* find and initialise all stations on this network */
    printf("Checking for ORL sensors in range %d - %d\n", 
	   MIN_STATION, MAX_STATION);

    if (getenv("POLLER_DIAGNOSE") != (char *) 0) {
        pollerDiagnose = TRUE;
    } else {
        pollerDiagnose = FALSE;
    }

    i = MIN_STATION;

    junk = serialTXRXByte(nPtr->fd, (unsigned char) STUFF);
    junk = serialTXRXByte(nPtr->fd, (unsigned char) (i & 0xff));

    for (++i; i <= MAX_STATION; i++) {

	nextRX = serialTXRXByte(nPtr->fd, STUFF);

	if (pollerDiagnose == TRUE) {
	    fprintf(stderr, "<%d>", (unsigned char) (i-1 & 0xff));
	    fflush(stderr);
	}

	if (nextRX == TIMEOUT) { /* station i doesn't exist */
	}
	else {
	    if (!(addStation(nPtr, (unsigned char) (i-1 & 0xff)))) {
		fprintf(stderr, "poller: failed to add station %d on network %d (%s)\n", i, nPtr->number, nPtr->name);
		exit(1); /* of course, maybe I won't exit here */
	    }
	    if (pollerDiagnose == TRUE) {
	        fprintf(stderr, "<Add Station %d>", (unsigned char) (i-1 & 0xff));
		fflush(stderr);
	    }
	}

	junk = serialTXRXByte(nPtr->fd, (unsigned char) (i & 0xff));
    }

    nextRX = serialTXRXByte(nPtr->fd, STUFF);

    if (nextRX == TIMEOUT) { /* station i doesn't exist */
    }
    else {
      if (!(addStation(nPtr, (unsigned char) (i & 0xff)))) { /* addStation also clears the FIFO */
	    fprintf(stderr, "poller: failed to add station %d on network %d (%s)\n"
		    , i, nPtr->number, nPtr->name);
	    exit(1);
	}
	
    }

    junk = serialTXRXByte(nPtr->fd, NULL_STATION);
    nPtr->lastStationAddressed = (StationList *) NULL;



    return(TRUE);
}

int
pollerInitialiseOlivetti(NetworkList *nPtr)
{
  int n,j;
  int res;
  unsigned char buf[255];
 


  printf("Checking for Olivetti sensors in range %d - %d\n", 
	 MIN_STATION, MAX_STATION);
  
  for(n = MIN_STATION ; n < MAX_STATION ; n++)
    {

      buf[0] = 0xFF;
      buf[1] = n;
      buf[2] = 0xE2;
      buf[3] = 0x01;
      buf[4] = 0x00;

      if( (res = serialTX(nPtr->fd,buf,5)) != 5)  
	{
	  printf("Can't send get_station_state command\n");
	  continue;
	}
      if( (res = serialRX(nPtr->fd,buf,255)) < 0)
	continue;
      

      //      for(j=0 ; j < 6 ; j++)
      //      	printf("%X ", 0xFF & buf[j]);
      //            printf("\n");

      if( buf[1] == 0x62)
	{
	  if (!(addStation(nPtr, (unsigned char) (n & 0xff)))) 
	    { 
	    fprintf(stderr, "poller: failed to add station %d on network %d (%s)\n"
		    , n, nPtr->number, nPtr->name);
	    exit(1); 
	    }
	}
       
      
    }

  nPtr->lastStationAddressed = (StationList *) NULL; 
   
  return(TRUE);
}



void 
SetPolls(int n)
{
  pthread_mutex_lock(&polls_m);
  _polls = n;
  pthread_mutex_unlock(&polls_m);
}

int 
GetPolls(void)
{
  int res;
  pthread_mutex_lock(&polls_m);
  res = _polls;
  pthread_mutex_unlock(&polls_m);
  return res;
}

void 
SetStations(int n)
{
  pthread_mutex_lock(&stations_m);
  _stations_total = n;
  pthread_mutex_unlock(&stations_m);
}

int 
GetStations(void)
{
  int res;
  pthread_mutex_lock(&stations_m);
  res = _stations_total;
  pthread_mutex_unlock(&stations_m);
  return res;
}
