/*
 * poller.h : code to poll a number of networks of active badges
 * 	      and report messages seen
 */

#ifndef POLLER_H
#define POLLER_H

#define POLLER_MESSAGE_BODY_MAX		(255)				/** Max. no of bytes in the body of a message **/

#define POLLER_MESSAGE_LENGTH		(4 + POLLER_MESSAGE_BODY_MAX)   /** Stn + Type + Length + Body + Network **/

#define MAX_TIMEOUTS	20
#define MAX_TIMEOUT_BLOCKS	5

#define	NULL_STATION	(unsigned char) 0x00

#define	NUMBER_OF_TYPES		256

#define POLL_UNKNOWN_INTERVAL 10

#define PROTO_OLIVETTI 1
#define PROTO_ORL      2
#define PROTO_UNKNOWN  100

typedef struct _PollerMessage {
    unsigned int        	count;
    unsigned char       	bytes[POLLER_MESSAGE_LENGTH];
    struct _PollerMessage  	*next;
} PollerMessage;

#define STRLEN		255

typedef struct _StationList {
    unsigned char	station;
    unsigned char	type;		/* latest type byte RXd */
    unsigned char	length;		/* latest length byte RXd */
    PollerMessage	message;	/* body of latest message RXd */
    int			RXing;		/* currently RXing from this stn */
    int			nextIsType;
    int			nextIsLength;
    int			lastWasStuff;
    int			timeouts;
    int			timeoutBlocks;	/* number of blocks of MAX_TIMEOUTS timeouts received
					   	(up to MAX_TIMEOUT_BLOCKS) */
    long		goodPacketsRXd;
    long		badPacketsRXd;
    long		totalPacketsTXd;
    long		startTime;
    PollerMessage	*firstTXMessage;/* next message to be transmitted */
    PollerMessage	*lastTXMessage; /* end of list of messages for TXsion*/
    struct _StationList	*next;
} StationList;

typedef struct _NetworkList {
    char		name[STRLEN];	/* device name */
    char                description[256];
    int                 protocol;       /* PROTO_OLIVETTI or PROTO_ORL */
    unsigned char	number;		/* number assigned to this device */
    int                 fd;             /* file descriptor */
    char		liveStations[256]; /* a record of live stations */
    StationList 	*stations;	/* stations active on this device */
    StationList		*lastStationAddressed; 
                                        /* last stn adrssd on this net */
    int			lastUnknownStation;
    long		goodPacketsRXd;
    long		badPacketsRXd;
    long		totalPacketsTXd;
    long		packetTypes[NUMBER_OF_TYPES];
    struct _NetworkList	*next;
} NetworkList;



int pollerAddSerialDevice(char *,char *descr, int protocol);

int pollerDeleteSerialDevice(char *);

int pollerInitialiseNetwork(char *);

PollerMessage *pollerRX(void);

int pollerTX(unsigned char, unsigned char, int, unsigned char *);

int pollerPollNetworks(NetworkList *nPtr);
int pollerPollOlivetti(NetworkList *nPtr);
int pollerPollORL(NetworkList *nPtr);

int pollerInitialiseORL(NetworkList *nPtr);
int pollerInitialiseOlivetti(NetworkList *nPtr);


void SetPolls(int n);
int GetPolls(void);
void SetStations(int n);
int GetStations(void);



#endif /* POLLER_H */
