//
// $Id: process_status.cc,v 1.1 1998/06/16 12:55:00 mencnar Exp $
//
// $Log: process_status.cc,v $
// Revision 1.1  1998/06/16 12:55:00  mencnar
// Initial revision
//
//
//-----------------------------------------------------------------------------
// 
// Author:      Daniel Mencnarowski  (mencnar@ics.agh.edu.pl)
//
// Description: Implemenation file
//              Implementation class for ABng_Server_Manager::ProcessControl
//


#include <iostream.h>
#include <strstream.h>
//#include "Naming.hh"
#include "process_status.hh"
#include <sys/types.h>
#include <sys/param.h>
#include <unistd.h>
#include <stdlib.h>
#include <netdb.h>
#include <pwd.h>


void ServerStatusInterface::get_server_status( ostream &ostr) {
  ostr << "Not implemented" << endl;
}

ProcessControlImpl::ProcessControlImpl( ABng_Server_Manager::Register_Manager_ptr srvman,
				        const char *_process_name,
					ServerStatusInterface *_status_impl) :
  status_impl( _status_impl),
  process_name( CORBA::string_dup( _process_name))
{
  gettimeofday( &start_time, NULL);
  CORBA::BOA::getBOA()->obj_is_ready( this);

  ABng_Server_Manager::Name	pol_name;
  pol_name.length(3);
  pol_name[0] = CORBA::string_dup("ABng");
  pol_name[1] = CORBA::string_dup("ps_dev");
  pol_name[2] = CORBA::string_dup(process_name);

  try
    {
      cout << "Registering server with Server Manager (psdev)" << endl;
      srvman->register_server( this,pol_name);
    }
  catch(ABng_Server_Manager::Server_Still_Active& e)
    {
      cout << "ServerManager exception: server still active" << endl;
      exit(1);
    }
  catch(ABng_Server_Manager::Invalid_Server_Name& e)
    {
      cout << "ServerManager exception: invalid server name" << endl;
      exit(1);
    }
  catch(ABng_Server_Manager::Invalid_Server_Reference& e)
    {
      cout << "ServerManager exception: invalid object reference" << endl;
      exit(1);
    }
  catch(CORBA::SystemException& e)
    {
      cout << "SystemException: " 
      << e.NP_RepositoryId() << endl;
      exit(1);
    }
  catch(omniORB::fatalException& e)
    {
      cout << "omniORB::fatalException: "
      << e.errmsg() << endl;
      exit(1);
    }
}

ProcessControlImpl::~ProcessControlImpl() {
  CORBA::string_free( process_name);
}

void time_bomb_wrapper( void *args) {
  sleep(2);
  exit(1);
}

void ProcessControlImpl::kill_server( ) {

  cerr << "ABng_Server::ProcessControl::kill_server called, exiting in 2 seconds";
  (new omni_thread( time_bomb_wrapper, NULL))->start();
}

CORBA::Long ProcessControlImpl::get_pid( ) {
  return (CORBA::Long) getpid();
}

char * ProcessControlImpl::get_host_name( ) {
  char buf[MAXHOSTNAMELEN];
  gethostname( buf, MAXHOSTNAMELEN);

  return CORBA::string_dup( buf);
}

char * ProcessControlImpl::get_process_owner( ) {
  struct passwd* pe= getpwuid( getuid());
  return CORBA::string_dup( pe ? pe->pw_name : "unknown");
}

CORBA::Long ProcessControlImpl::uptime( ) {
  struct timeval now;
  gettimeofday( &now, NULL);
  return now.tv_sec - start_time.tv_sec;
}

char * ProcessControlImpl::get_server_status( ) {
  if ( status_impl) {
    ostrstream ostr;   
    status_impl->get_server_status( ostr);
    ostr << '\0'; // terminate string
    char *str= ostr.str();
    char *ret= CORBA::string_dup( str);
    free(str);
    return ret;
  }
  else return CORBA::string_dup( "Not implemented\n");
}







