#include <iostream.h>
#include "sabpoller_i.hh"
#include <pthread.h>
#include <fcntl.h>
#include <stdlib.h>
#include "socket.h"
#include "abngpoller.hh"

extern CORBA::ORB_ptr  orb;
extern CORBA::BOA_ptr  boa;
extern char*	       serial_device;

SABobserver* SABobserver::observer_head = NULL;
SABobserver* SABobserver::observer_tail = NULL;

pthread_mutex_t read_mutex[255] = {PTHREAD_MUTEX_INITIALIZER};

SABobserver*
SABobserver::get_observer_head()
{
     return observer_head;
}

SABobserver*
SABobserver::get_observer_tail()
{
     return observer_tail;
}

SABobserver::SABobserver()
{
     observer = NULL;
     simple_observer = NULL;
     extended_observer = NULL;
     next = NULL;
     type = 0;
     obs_type = SABobserver::NO_OBSERVER;
}

SABobserver::SABobserver(ABng_Poller::AB_Observer_ptr obs, unsigned char typ)
{
     observer = obs;
     simple_observer = (ABng_Poller::Simple_AB_Observer_ptr)obs;
     obs_type = SABobserver::STANDARD;
     type = typ;
     next = NULL;
     if (observer_head == NULL)
	  observer_head = observer_tail = this;
     else
     {
	  observer_tail->next = this;
	  observer_tail = this;
     }
}

SABobserver::SABobserver(ABng_Poller::Simple_AB_Observer_ptr obs, unsigned char typ)
{
     observer = NULL;
     simple_observer = obs;
     obs_type = SABobserver::SIMPLE;
     type = typ;
     next = NULL;
     if (observer_head == NULL)
	  observer_head = observer_tail = this;
     else
     {
	  observer_tail->next = this;
	  observer_tail = this;
     }
}

SABobserver::SABobserver(ABng_Poller::AB_Observer_V2_ptr obs, unsigned char typ)
{
     observer = (ABng_Poller::AB_Observer_ptr)obs;
     simple_observer = (ABng_Poller::Simple_AB_Observer_ptr)obs;
     extended_observer = obs;
     obs_type = SABobserver::EXTENDED;
     type = typ;
     next = NULL;
     if (observer_head == NULL)
	  observer_head = observer_tail = this;
     else
     {
	  observer_tail->next = this;
	  observer_tail = this;
     }
}

SABobserver*
SABobserver::get_next()
{
     return next;
}

void
SABobserver::set_next(SABobserver* obs)
{
     next = obs;
}

ABng_Poller::AB_Observer_ptr
SABobserver::get_observer()
{
     return observer;
}

void
SABobserver::set_observer(ABng_Poller::AB_Observer_ptr obs)
{
     observer = obs;
     simple_observer = (ABng_Poller::Simple_AB_Observer_ptr)obs;
}

ABng_Poller::AB_Observer_V2_ptr
SABobserver::get_extended_observer()
{
     return extended_observer;
}

void
SABobserver::set_extended_observer(ABng_Poller::AB_Observer_V2_ptr obs)
{
     extended_observer = obs;
     observer = (ABng_Poller::AB_Observer_ptr)obs;
     simple_observer = (ABng_Poller::Simple_AB_Observer_ptr)obs;
}

ABng_Poller::Simple_AB_Observer_ptr
SABobserver::get_simple_observer()
{
     return simple_observer;
}

void
SABobserver::set_simple_observer(ABng_Poller::Simple_AB_Observer_ptr obs)
{
     simple_observer = obs;
}

unsigned char
SABobserver::get_type()
{
     return type;
}

void
SABobserver::set_type(unsigned char typ)
{
     type = typ;
}

SABobserver::observer_type
SABobserver::get_obs_type()
{
     return obs_type;
}

void
SABobserver::set_obs_type(SABobserver::observer_type typ)
{
     obs_type = typ;
}

void
SABobserver::add_to_list()
{
     if (observer_head == NULL)
	  observer_head = observer_tail = this;
     else
     {
	  observer_tail->next = this;
	  observer_tail = this;
     }
}

int
SABobserver::remove_from_list(ABng_Poller::Simple_AB_Observer_ptr obs,unsigned char type)
{
     SABobserver        *temp_obs1 = NULL,*temp_obs2 = NULL;
     int		found = 0;
     char		ref1[500];
     char		ref2[500];

     /* Can't simply compare the references got by "string_to_object"
	because of strange behavior of this function. Have to copy them
	to separate strings first */

     strcpy(ref1,orb->object_to_string(obs));
     temp_obs1 = observer_head;
     while(temp_obs1 != NULL)
     {
	  switch (temp_obs1->get_obs_type())
	  {
	  case SABobserver::SIMPLE:
	       strcpy(ref2,orb->object_to_string(temp_obs1->get_simple_observer()));
	       break;

	  case SABobserver::STANDARD:
	       strcpy(ref2,orb->object_to_string((ABng_Poller::Simple_AB_Observer_ptr)temp_obs1->get_observer()));
	       break;

	  case SABobserver::EXTENDED:
	       strcpy(ref2,orb->object_to_string((ABng_Poller::Simple_AB_Observer_ptr)temp_obs1->get_extended_observer()));
	       break;

	  default:
	       cout << "Unknown observer type!!!" << endl;
	       break;
	  }

	  if ((type == 0 || temp_obs1->get_type() == type) &&
	      !strcmp(ref1,ref2))
	  {
	       found = 1;
	       if (temp_obs1 == observer_head)
	       {
		    observer_head = temp_obs1->get_next();
	       }
	       else
	       {
		 // !!!
		    temp_obs2->set_next(temp_obs1->get_next());
	       }
	       if (temp_obs1 == observer_tail)
	       {
		    observer_tail = temp_obs2;
	       }
	       switch (temp_obs1->get_obs_type())
	       {
	       case SABobserver::STANDARD:
		    CORBA::release(temp_obs1->get_observer());
		    break;

	       case SABobserver::SIMPLE:
		    CORBA::release(temp_obs1->get_simple_observer());
		    break;

	       case SABobserver::EXTENDED:
		    CORBA::release(temp_obs1->get_extended_observer());
		    break;

	       default:
		    cout << "Unknown observer type" << endl;
		    break;
	       }
	       delete temp_obs1;
	       break;
	  }
	  temp_obs2 = temp_obs1;
	  temp_obs1 = temp_obs1->get_next();
     }
     if (!found) 
     {
	  cout << "Can't find this observer..." << endl;
	  return -1;
     }
     else 
     {
	  cout << "Observer detached" << endl;
	  return 0;
     }
}

/************************Simple_AB_Poller*************************/

CORBA::Boolean Simple_AB_Poller_i::_debug = 0;

Simple_AB_Poller_i::Simple_AB_Poller_i()
{
     int i;

     _click_counting_flag = 0;
     _max_clicks = 1;
     _click_timeout = 1;
     _delay = 19;
     _name = strdup("Not named");
     for (i=0;i<255;i++)
     {
       _networks[i] = Simple_AB_Poller_i::ORL; // IB
	  _serials[i] = NULL;
     }
     _sensor_timeout = 0;
}

void
Simple_AB_Poller_i::sensor_timeout(int sens)
{
     _sensor_timeout = sens;
}

int 
Simple_AB_Poller_i::sensor_timeout()
{
     return _sensor_timeout;
}

void
Simple_AB_Poller_i::set_serial_name(unsigned char network_nr, char* name)
{
     if (_serials[network_nr]) delete [] _serials[network_nr];
     _serials[network_nr] = strdup(name);
}

char*
Simple_AB_Poller_i::get_serial_name(unsigned char network_nr)
{
     return _serials[network_nr];
}

void
Simple_AB_Poller_i::set_network_type(unsigned char network_nr, Simple_AB_Poller_i::NetworkType type)
{
     _networks[network_nr] = type;
}

Simple_AB_Poller_i::NetworkType
Simple_AB_Poller_i::get_network_type(unsigned char network_nr)
{
     return _networks[network_nr];
}

CORBA::Boolean
Simple_AB_Poller_i::click_counting_flag()
{
     return _click_counting_flag;
}

void
Simple_AB_Poller_i::click_counting_flag(CORBA::Boolean _value)
{
     _click_counting_flag = _value;
     clicks::click_counting_flag = _value;
}

char*
Simple_AB_Poller_i::name()
{
     return _name;
}

void
Simple_AB_Poller_i::name(const char* _value)
{
     _name = CORBA::string_dup(_value);
     //     cout << "Name = " << _value << endl;
}

unsigned short
Simple_AB_Poller_i::max_clicks()
{
     return _max_clicks;
}

void
Simple_AB_Poller_i::max_clicks(unsigned short aaa)
{
     _max_clicks = aaa;
     clicks::max_clicks = aaa;
}

unsigned short
Simple_AB_Poller_i::click_timeout()
{
     return _click_timeout;
}

void
Simple_AB_Poller_i::click_timeout(unsigned short aaa)
{
     _click_timeout = aaa;
     clicks::click_timeout = aaa;
}

unsigned short
Simple_AB_Poller_i::delay()
{
     return _delay;
}

void
Simple_AB_Poller_i::delay(unsigned short aaa)
{
     _delay = aaa;
}

void
Simple_AB_Poller_i::debug(CORBA::Boolean val)
{
     _debug = val;
}

CORBA::Boolean
Simple_AB_Poller_i::debug()
{
     return _debug;
}

CORBA::Boolean 
Simple_AB_Poller_i::attach_observer(ABng_Poller::Simple_AB_Observer_ptr obs)
{
     return attach_type_observer(obs,0);
}

CORBA::Boolean 
Simple_AB_Poller_i::detach_observer(ABng_Poller::Simple_AB_Observer_ptr obs)
{
     return detach_type_observer(obs,0);
}

CORBA::Boolean
Simple_AB_Poller_i::attach_type_observer(ABng_Poller::Simple_AB_Observer_ptr obs, CORBA::Short type)
{
     ABng_Poller::AB_Observer_ptr temp_obs;
     ABng_Poller::AB_Observer_V2_ptr temp_obs_v2;

     if (CORBA::is_nil(obs)) 
     {
	  if (_debug) cerr << "Simple AB Poller: cannot invoke on a nil object reference" << endl;
	  return 1;
     }
     temp_obs_v2 = ABng_Poller::AB_Observer_V2::_narrow(obs);
     if (!CORBA::is_nil(temp_obs_v2)) 
     {
	  ABng_Poller::AB_Observer_V2::_duplicate(temp_obs_v2);
	  SABobserver*	new_obs = new SABobserver(temp_obs_v2,type);
     }
     else
     {
	  temp_obs = ABng_Poller::AB_Observer::_narrow(obs);
	  if (!CORBA::is_nil(temp_obs)) 
	  {
	       ABng_Poller::AB_Observer::_duplicate(temp_obs);
	       SABobserver*	new_obs = new SABobserver(temp_obs,type);
	  }
	  else
	  {
	       ABng_Poller::Simple_AB_Observer::_duplicate(obs);
	       SABobserver*	new_obs = new SABobserver(obs,type);
	  }
     }
     if (_debug) cout << "New observer (type " << type << ") attached" << endl;
     return 0;
}

CORBA::Boolean
Simple_AB_Poller_i::detach_type_observer(ABng_Poller::Simple_AB_Observer_ptr obs, CORBA::Short type)
{
     return SABobserver::remove_from_list(obs,type);
}

CORBA::Boolean
Simple_AB_Poller_i::detach_observer_all(ABng_Poller::Simple_AB_Observer_ptr obs)
{
     detach_type_observer(obs,0);
     return 0;
}

void
Simple_AB_Poller_i::issue_command(const ABng_Poller::Simple_AB_Poller::Message& cmd)
{
  unsigned int i;
  SocketMessage *msg;


  msg = (SocketMessage*) malloc(sizeof(SocketMessage));
  if(!msg)
    {
      cout << "malloc failed in Simple_AB_Poller_i::issue_command\n";
      return;
    }

  msg->socket  = 0;
  msg->station = cmd[1];
  msg->network = cmd[cmd.length()-1];
  msg->length = cmd.length();
  msg->next = (SocketMessage *) NULL;
  for(i=0 ; i < cmd.length() ; i++)
    msg->body[i] = cmd[i];
 
  CorbaAddMessage(msg);


  //     int		fd;
//      termios		oldtio;
//      int		res,i;
//      unsigned char*	command;

//      //if (_debug) cout << "<<issue_command>> called" << endl;

//      if ((fd = open_device(_serials[cmd[cmd.length()-1]])) < 0)
//      {
// 	  if (_debug) cout << "Can't open serial device!!" << endl;
// 	  return;
//      }
     
//      if (cmd.length() == 0)
//      {
// 	  if (_debug) cout << "Can't send empty message" << endl;
// 	  return;
//      }

//      command = new unsigned char [cmd.length()];
//      for(i=0;i<(int)cmd.length()-1;i++)
//      {
// 	  command[i] = cmd[i];
//      }
//      command[0] = 0xFF;
//      //if (_debug) cout << "Sending a command..." << endl;
//      pthread_mutex_lock(&read_mutex[cmd[cmd.length()-1]]);
//      res = write(fd, command, cmd.length()-1);
//      pthread_mutex_unlock(&read_mutex[cmd[cmd.length()-1]]);
//      delete [] command;

//      if(res != (int)cmd.length()-1)
//      {
// 	  perror("Error while sending a command");
// 	  exit(-1);
//      }
//      //if (_debug) cout << "Sending completed" << endl;
//      close_device(fd);
//      return;
}

int
Simple_AB_Poller_i::open_device(char* serial_dev)
{
     int	fd;
     termios  newtio,oldtio;

     fd = open(serial_dev, O_RDWR|O_NONBLOCK);
     if (fd<0) 
     {
	  perror(serial_dev);
	  return fd;
     }
     tcgetattr(fd,&oldtio);
     bzero(&newtio,sizeof(newtio));

     newtio.c_cflag = BAUDRATE | CS8 | CREAD | CLOCAL | CSTOPB ;
     newtio.c_iflag = 0;    /* raw input */
     newtio.c_oflag = 0;    /* raw output */
     newtio.c_lflag = 0;    /* raw line connection */
     newtio.c_cc[VTIME] = 0;
     newtio.c_cc[VMIN]  = 1;
     
     tcflush(fd,TCIFLUSH);
     tcsetattr(fd,TCSANOW,&newtio);
     return fd;
}

void
Simple_AB_Poller_i::close_device(int fd)
{
     //tcsetattr(fd,TCSANOW,&oldtio);
     close(fd);
     return;
}

/*********************************clicks*******************************/
clicks*		clicks::click_head = NULL;
CORBA::Boolean	clicks::click_counting_flag = 0;
unsigned short	clicks::max_clicks = 1;
unsigned short	clicks::click_timeout = 10;
pthread_mutex_t clicks::click_mutex = PTHREAD_MUTEX_INITIALIZER;

clicks::clicks(ABng_Poller::AB_Poller::Badge_Id badge_par, timeval tv, buttons butt, ABng_Poller::AB_Poller::Message mess, int sens_mark)
{
     clicks*	temp_click;

     badge = badge_par;
     pressed = 1;
     timestamp.tv_sec = tv.tv_sec;
     timestamp.tv_usec = tv.tv_usec;
     button = butt;
     msg = mess;
     sensor_mark = sens_mark;
     next = NULL;
     if (click_head == NULL) click_head = this;
     else
     {
	  temp_click = click_head;
	  while(temp_click->next) temp_click = temp_click->next;
	  temp_click->next = this;
     }
}

unsigned short
clicks::send_it(ABng_Poller::AB_Poller::Badge_Id badge_par, timeval ts, buttons butt, ABng_Poller::AB_Poller::Message mess, int sens_mark)
{
     clicks*	temp_click1,*temp_click2;

     pthread_mutex_lock(&click_mutex);
     if (!click_counting_flag) 
     {
	  pthread_mutex_unlock(&click_mutex);
	  return max_clicks;
     }

     temp_click1 = temp_click2 = click_head;
     while(temp_click1)
     {
	  if ((temp_click1->compare_badge_id(badge_par)) && 
	      (temp_click1->button == butt) &&
	      (temp_click1->sensor_mark == sens_mark))
	  {
	       if (++(temp_click1->pressed) < max_clicks)
	       {
		    temp_click1->timestamp.tv_usec = ts.tv_usec;
		    temp_click1->timestamp.tv_sec = ts.tv_sec; 
		    cout << "Incrementing <<pressed>>" << endl;
		    pthread_mutex_unlock(&click_mutex);
		    return 0;
	       }
	       else
	       {
		    cout << "Enough... sending" << endl;
		    if (temp_click1 == click_head)
		    {
			 click_head = click_head->next;
			 delete temp_click1;
		    }
		    else
		    {
			 temp_click2->next = temp_click1->next;
			 delete temp_click1;
		    }
		    pthread_mutex_unlock(&click_mutex);
		    return max_clicks;
	       }
	  }
	  temp_click2 = temp_click1;
	  temp_click1 = temp_click1->next;
     }

     if (max_clicks == 1) 
     {
	  pthread_mutex_unlock(&click_mutex);
	  return max_clicks;
     }
     else
     {
	  cout << "First click..." << endl;
	  clicks*	new_click = new clicks(badge_par,ts,butt,mess,sens_mark);
	  pthread_mutex_unlock(&click_mutex);
	  return 0;
     }
}

unsigned short
clicks::get_outdated(ABng_Poller::AB_Poller::Message& mess, int& sens_mark, timeval ts)
{
     clicks*		temp_click1,*temp_click2;
     unsigned short	return_clicks;

     pthread_mutex_lock(&click_mutex);
     temp_click1 = temp_click2 = click_head;
     while(temp_click1)
     {
	  if ((ts.tv_sec - temp_click1->timestamp.tv_sec)*1000000 + 
	      (ts.tv_usec - temp_click1->timestamp.tv_usec) > 
	      click_timeout * 100000)
	  {
	       /* click outdated */
	       cout << "click outdated" << endl;
	       if (temp_click1 == click_head)
	       {
		    click_head = click_head->next;
		    return_clicks = temp_click1->pressed;
		    mess = temp_click1->msg;
		    sens_mark = temp_click1->sensor_mark;
		    delete temp_click1;
		    pthread_mutex_unlock(&click_mutex);
		    return return_clicks;
	       }
	       else
	       {
		    temp_click2->next = temp_click1->next;
		    return_clicks = temp_click1->pressed;
		    mess = temp_click1->msg;
		    sens_mark = temp_click1->sensor_mark;
		    delete temp_click1;
		    pthread_mutex_unlock(&click_mutex);
		    return return_clicks;
	       }
	  }
	  temp_click2 = temp_click1;
	  temp_click1 = temp_click1->next;
     }
     pthread_mutex_unlock(&click_mutex);
     return 0;
}

CORBA::Boolean
clicks::compare_badge_id(ABng_Poller::AB_Poller::Badge_Id bad)
{
     unsigned int	i;
     
     if (badge.length() == bad.length())
     {
	  for (i=0;i<badge.length();i++)
	       if (badge[i] != bad[i]) return 0;
	  return 1;
     }
     return 0;
}

