/*
 * serialDevice.c : unix serial device control for use with abpoll,
 * 		    a new active badge station network poller.
 */

#include <stdio.h>
#include <sys/types.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <termio.h>
#include <fcntl.h>

#ifndef TRUE
#define TRUE            1
#endif

#ifndef FALSE
#define FALSE           0
#endif

#define AB_TX_RX_GAP_IN_MS         70000 /* 5 */ /* was 70000 */

#define MIN_CHARS               1 /* roy uses 0 */
#define STRLEN			255



#include "serial.h"
#include "globals.h"

typedef struct _SerialDevice {
    char			*name;
  //    int				number;
    int fd;
    struct _SerialDevice	*next;
} SerialDevice;


extern int NextNumber;

static SerialDevice		*serialDevices = (SerialDevice *) NULL;


int
serialRX(int fd, unsigned char *buf, int len)
{
  struct timeval tm;
  int  nfds, readfds, exceptfds, result;
  tm.tv_sec = 0;
  tm.tv_usec = AB_TX_RX_GAP_IN_MS ;

  nfds = fd + 1;
  readfds = exceptfds = 1 << fd;

  if (select(nfds, (fd_set*) &readfds, 0, (fd_set*) &exceptfds, &tm) > 0) {
    if (readfds & (1 << fd)) {

      if ((result = read(fd, buf, len)) > 0) {
	return result;
		
      }
      else return 0;
    }
  }


}

int
serialTX(int fd, unsigned char *buf, int len)
{

  return write(fd , buf , len);

}



/*
 * ---- Transmit and Receive a byte from a specified device
 */
int 
serialTXRXByte(deviceNumber, byte)
    int		deviceNumber;
    unsigned char	byte;
{
    SerialDevice	*ptr;
    int			n, nfds, readfds, exceptfds;
    struct timeval      tm;
    static int		diagnose = FALSE;

    //    deviceNumber += 3;
 
    if (diagnose) fprintf(stderr, "%02x  ", byte);

    
    if (write(deviceNumber, &byte, 1) <= 0) {
        fprintf(stderr, "serialTXRXByte: write failed for character %02x , dev %d\n",byte,deviceNumber);
	exit(1);
    }

    tm.tv_sec = 0;
    tm.tv_usec = AB_TX_RX_GAP_IN_MS ;

    nfds = deviceNumber + 1;
    readfds = exceptfds = 1 << deviceNumber;

    if (select(nfds, (fd_set*) &readfds, 0, (fd_set*) &exceptfds, &tm) > 0) {
        if (readfds & (1 << deviceNumber)) {

	    if (read(deviceNumber, &byte, 1) > 0) {
		if (diagnose) fprintf(stderr, "%02x \n", byte);
		
	        return byte & 0xff;
	    }
	}
    }
   
    if (diagnose) fprintf(stderr, "TO \n");

    return TIMEOUT;
}


int
serialOpen(deviceName)
    char	*deviceName;
{
    SerialDevice	*ptr, *new;
    struct termio  	t;



    if ((deviceName == (char *) NULL) || (strlen(deviceName) == 0)) {
	return -1;
    }

    for (ptr = serialDevices; ptr != (SerialDevice *) NULL; ptr = ptr->next) {
	if (strcmp(deviceName, ptr->name) == 0) {
	    break;
	}
    }


    if (ptr != (SerialDevice *) NULL) {
	fprintf(stderr, "serialOpen: warning: \"%s\" already exists\n", deviceName);
	return TRUE;
    }

    if (!(new = (SerialDevice *) malloc (sizeof(SerialDevice)))) {
	fprintf(stderr, "serialOpen: malloc failure\n");
	exit(1);
    }

    if (!(new->name = (char *) malloc (sizeof(char) * (strlen(deviceName) + 1)))) {
	fprintf(stderr, "serialOpen: malloc failure\n");
	exit(1);
    }
	
    strcpy(new->name, deviceName);

    if ((new->fd = open(new->name, O_RDWR, 0)) < 0) {
        fprintf(stderr, "serialOpen : open failed\n");
	free ((char *) new->name);
	free ((char *) new);
	return -1;
    }

    if (ioctl(new->fd, TCGETA, &t) < 0) {
        fprintf(stderr, "serialOpen : TCGETA failed\n");
	free ((char *) new->name);
	free ((char *) new);
	return -1;
    }

    t.c_iflag = 0; 	/* raw input */
    t.c_oflag = 0; 	/* raw output */
    t.c_lflag = 0;    	/* raw line connection*/

   /* hardware: 2 stop bits, no parity, rx enabled, 8 data bits, 9600 baud */
    t.c_cflag = B9600 | CS8 | CREAD | CLOCAL | CSTOPB; 

    t.c_cc[VMIN] = (unsigned char) MIN_CHARS;
    t.c_cc[VTIME] = (unsigned char) 0; /** 0 timeout **/

    if (ioctl(new->fd, TCSETA, &t)< 0) {
	perror("serialDevice ioctl TCSETA failed");
	free ((char *) new->name);
	free ((char *) new);
	return -1;
    }

    new->next = serialDevices;
    serialDevices = new;
    ++NextNumber;
    
    return new->fd ;
}

int
serialClose(deviceName)
    char	*deviceName;
{
    SerialDevice	**list, *ptr;

#undef IMPLEMENTED
#ifdef IMPLEMENTED

    if ((deviceName == (char *) NULL) || (strlen(deviceName) == 0)) {
	return FALSE; /* silly device */
    }

    list = &serialDevices;
    ptr = (SerialDevice *) NULL;
    while (*list) {
	if (strcmp(deviceName, (*list)->name) == 0) {
	    ptr = *list;
	    *list = (*list)->next;
	    break;
	}
	else {
	    list = &((*list)->next);
	}
    }

    if (ptr == (SerialDevice *) NULL) {
	return FALSE; /* unknown device */
    }

    if (close(ptr->fd) != 0) {
	char	report[STRLEN];
	sprintf(report, "serialClose: failed to close \"%s\"\n", deviceName);
	perror(report);
	return FALSE;
    }

    free ((char *) ptr->name);
    free ((char *) ptr);

    return TRUE;
#endif
    return FALSE;
}



